"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.build_docs_project = void 0;
const jsonschema2markdown_1 = require("@doc-utils/jsonschema2markdown");
const markdown2html_1 = require("@doc-utils/markdown2html");
const glob_1 = require("glob");
const luxon_1 = require("luxon");
const yaml_1 = require("yaml");
const fs_1 = require("fs");
const path_1 = require("path");
const icons_1 = require("./icons");
const env_1 = require("./env");
const themes_1 = require("./themes");
const fs_2 = require("./fs");
const template_1 = require("./template");
async function build_docs_project(conf) {
    const now = luxon_1.DateTime.now();
    const themes = await (0, themes_1.load_themes)(conf);
    const state = {
        conf,
        seen_files: new Set(),
        env: (0, env_1.build_env_scope)(conf),
        layouts: Object.create(null),
        themes: themes,
        theme_groups: {
            // fixme: this is horribly inefficient
            all: Object.values(themes),
            light: Object.values(themes).filter((theme) => theme.labels.includes('light')),
            dark: Object.values(themes).filter((theme) => theme.labels.includes('dark')),
            high_contrast: Object.values(themes).filter((theme) => theme.labels.includes('high_contrast')),
            low_contrast: Object.values(themes).filter((theme) => theme.labels.includes('low_contrast')),
            monochrome: Object.values(themes).filter((theme) => theme.labels.includes('monochrome')),
            greyscale: Object.values(themes).filter((theme) => theme.labels.includes('greyscale')),
            protanopia_safe: Object.values(themes).filter((theme) => theme.labels.includes('protanopia_safe')),
            deuteranopia_safe: Object.values(themes).filter((theme) => theme.labels.includes('deuteranopia_safe')),
            tritanopia_safe: Object.values(themes).filter((theme) => theme.labels.includes('tritanopia_safe')),
        },
        extras: await (0, template_1.load_extras)(),
        made_directories: new Set(),
        build_time: {
            iso: now.toISO(),
            rfc2822: now.toRFC2822(),
        },
    };
    if (conf.input.raw) {
        await copy_raw_files(state);
    }
    if (conf.input.text) {
        await render_text_file_templates(state);
    }
    if (conf.input.markdown) {
        await render_markdown_files(state);
    }
    if (conf.input['schema+json'] || conf.input['schema+yaml']) {
        await render_json_schema_files(state);
    }
    // todo...
}
exports.build_docs_project = build_docs_project;
// ===== Raw File Copy =====
async function copy_raw_files(state) {
    const promises = [];
    const files = await (0, glob_1.glob)(state.conf.input.raw, {
        absolute: true,
        cwd: state.conf.input.root,
    });
    for (const in_file of files) {
        if (state.seen_files.has(in_file)) {
            continue;
        }
        state.seen_files.add(in_file);
        const out_file = map_input_file_to_output_file(state, in_file);
        promises.push(fs_1.promises.copyFile(in_file, await out_file));
    }
    await Promise.all(promises);
}
// ===== File Renderers =====
async function render_text_file_templates(state) {
    const promises = [];
    const files = await (0, glob_1.glob)(state.conf.input.text, {
        absolute: true,
        cwd: state.conf.input.root,
    });
    if (!state.partials) {
        await build_partials(state);
    }
    for (const in_file of files) {
        if (state.seen_files.has(in_file)) {
            continue;
        }
        state.seen_files.add(in_file);
        promises.push(render_text_file_template(state, in_file));
    }
    await Promise.all(promises);
}
async function render_text_file_template(state, in_file) {
    const out_file = map_input_file_to_output_file(state, in_file);
    const { frontmatter, text } = await (0, fs_2.read_text)(in_file);
    let layout;
    const layout_file = frontmatter?.layout;
    if (layout_file) {
        if (!state.layouts[layout_file]) {
            state.layouts[layout_file] = await (0, template_1.load_layout)(state.conf, layout_file);
        }
        layout = state.layouts[layout_file];
    }
    const tags = state.conf.templates?.tags;
    const context = mustache_context(state, frontmatter);
    const rendered = (0, template_1.render_template)(text, context, layout, structuredClone(state.partials), tags);
    await (0, fs_2.write_text)(await out_file, rendered);
}
async function render_markdown_files(state) {
    const promises = [];
    const files = await (0, glob_1.glob)(state.conf.input.markdown, {
        absolute: true,
        cwd: state.conf.input.root,
    });
    if (!state.partials) {
        await build_partials(state);
    }
    for (const in_file of files) {
        if (state.seen_files.has(in_file)) {
            continue;
        }
        state.seen_files.add(in_file);
        promises.push(render_markdown_file(state, in_file));
    }
    await Promise.all(promises);
}
async function render_markdown_file(state, in_file) {
    const out_file = map_input_file_to_output_file(state, in_file, ['.md', '.markdown'], '.html');
    const { frontmatter, text } = await (0, fs_2.read_text)(in_file);
    const html = (0, markdown2html_1.render_markdown_to_html)(text, state.conf.markdown);
    let layout;
    const layout_file = frontmatter?.layout;
    if (layout_file) {
        if (!state.layouts[layout_file]) {
            state.layouts[layout_file] = await (0, template_1.load_layout)(state.conf, layout_file);
        }
        layout = state.layouts[layout_file];
    }
    const tags = state.conf.templates?.tags;
    const context = mustache_context(state, frontmatter);
    const rendered = (0, template_1.render_template)(await html, context, layout, structuredClone(state.partials), tags);
    await (0, fs_2.write_text)(await out_file, rendered);
}
async function render_json_schema_files(state) {
    const promises = [];
    const json_files = await (0, glob_1.glob)(state.conf.input['schema+json'], {
        absolute: true,
        cwd: state.conf.input.root,
    });
    const yaml_files = await (0, glob_1.glob)(state.conf.input['schema+yaml'], {
        absolute: true,
        cwd: state.conf.input.root,
    });
    if (!state.partials) {
        await build_partials(state);
    }
    for (const in_file of json_files) {
        if (state.seen_files.has(in_file)) {
            continue;
        }
        state.seen_files.add(in_file);
        promises.push(handle_json_schema_json_file(state, in_file));
    }
    for (const in_file of yaml_files) {
        if (state.seen_files.has(in_file)) {
            continue;
        }
        state.seen_files.add(in_file);
        promises.push(handle_json_schema_yaml_file(state, in_file));
    }
    await Promise.all(promises);
}
async function handle_json_schema_json_file(state, in_file) {
    const promises = [];
    const out_file = map_input_file_to_output_file(state, in_file, ['.json'], '.html');
    const { frontmatter, parsed, json } = await (0, fs_2.read_json)(in_file, true);
    promises.push(render_json_schema(state, parsed, await out_file, frontmatter));
    if (state.conf.output.include_inputs?.includes('schema+json')) {
        const json_file = await map_input_file_to_output_file(state, in_file);
        promises.push((0, fs_2.write_text)(json_file, json));
        if (state.conf.output.include_yaml_and_json) {
            const yaml_file = await map_input_file_to_output_file(state, in_file, ['.json'], '.yaml');
            promises.push((0, fs_2.write_text)(yaml_file, (0, yaml_1.stringify)(parsed)));
        }
    }
    await Promise.all(promises);
}
async function handle_json_schema_yaml_file(state, in_file) {
    const promises = [];
    const out_file = map_input_file_to_output_file(state, in_file, ['.yaml', '.yml'], '.html');
    const { frontmatter, parsed, yaml } = await (0, fs_2.read_yaml)(in_file, true);
    promises.push(render_json_schema(state, parsed, await out_file, frontmatter));
    if (state.conf.output.include_inputs?.includes('schema+yaml')) {
        const yaml_file = await map_input_file_to_output_file(state, in_file);
        promises.push((0, fs_2.write_text)(yaml_file, yaml));
        if (state.conf.output.include_yaml_and_json) {
            const json_file = await map_input_file_to_output_file(state, in_file, ['.yaml', '.yml'], '.json');
            promises.push((0, fs_2.write_text)(json_file, JSON.stringify(parsed, null, '  ')));
        }
    }
    await Promise.all(promises);
}
async function render_json_schema(state, schema, out_file, frontmatter) {
    const promises = [];
    const markdown = (0, jsonschema2markdown_1.build_markdown_from_json_schema)(schema);
    if (state.conf.output.include_intermediate_markdown) {
        promises.push(map_input_file_to_output_file(state, out_file, ['.html'], '.md')
            .then((md_file) => (0, fs_2.write_text)(md_file, markdown)));
    }
    const html = (0, markdown2html_1.render_markdown_to_html)(markdown, state.conf.markdown);
    let layout;
    const layout_file = frontmatter?.layout;
    if (layout_file) {
        if (!state.layouts[layout_file]) {
            state.layouts[layout_file] = await (0, template_1.load_layout)(state.conf, layout_file);
        }
        layout = state.layouts[layout_file];
    }
    const tags = state.conf.templates?.tags;
    const context = mustache_context(state, frontmatter);
    const rendered = (0, template_1.render_template)(await html, context, layout, structuredClone(state.partials), tags);
    promises.push((0, fs_2.write_text)(await out_file, rendered));
    await Promise.all(promises);
}
// ===== Helpers =====
async function map_input_file_to_output_file(state, in_file, remove_exts, add_ext) {
    if (!in_file.startsWith(state.conf.input.root)) {
        throw new Error('input file expected to be inside input root');
    }
    let out_file = (0, path_1.join)(state.conf.output.root, in_file.slice(state.conf.input.root.length));
    if (remove_exts) {
        for (const ext of remove_exts) {
            if (out_file.endsWith(ext)) {
                out_file = out_file.slice(0, -ext.length);
                break;
            }
        }
    }
    if (add_ext) {
        out_file += add_ext;
    }
    const dir = (0, path_1.dirname)(out_file);
    if (!state.made_directories.has(dir)) {
        state.made_directories.add(dir);
        await (0, fs_2.mkdirp)(dir);
    }
    return out_file;
}
async function build_partials(state) {
    state.partials = await (0, template_1.load_partials)(state.conf);
    for (const [name, theme] of Object.entries(state.themes)) {
        state.partials[`.themes/${name}`] = (0, themes_1.render_theme_css_properties)(theme);
    }
    Object.assign(state.partials, state.extras);
}
function mustache_context(state, frontmatter) {
    return {
        env: state.env,
        page: frontmatter,
        build_time: state.build_time,
        icons: icons_1.icons,
        themes: Object.values(state.themes),
        theme_groups: structuredClone(state.theme_groups),
        markdown: {
            render_inline() {
                return (text, render) => {
                    const md = render(text);
                    const html = (0, markdown2html_1.render_markdown_to_html_inline_sync)(md, state.conf.markdown);
                    return html;
                };
            },
        }
    };
}
//# sourceMappingURL=build.js.map